import 'dart:math' as math;

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:knt_bloc/knt_bloc.dart';

import '../../bloc/paging_gallery_bloc.dart';
import '../../defined.dart';
import '../../model/models.dart';
import '../image/knt_cached_image.dart';
import '../image/knt_local_image.dart';
import '../knt_ripple.dart';
import 'knt_gallery_page.dart';
import 'knt_photo_browser_page.dart';

const _defaultStyleCount = 2;

/// Staggered Gallery with default 2 styles
///
/// Photos will be separated into blocks, each block has [numberPhotoInBlock] photos
class KntStaggeredGallery extends StatelessWidget {
  final List<Photo> photoList;
  final int maxPresentPhoto;
  final int numberPhotoInBlock;
  final Axis direction;
  final double? mainSpacing;
  final double? crossSpacing;
  final EdgeInsets? padding;
  final Function(int)? onPickPhoto;
  final VoidCallback? onSeeAll;

  /// Convert number of hidden photos into text
  final String Function(int)? convertedSeeAllText;

  const KntStaggeredGallery({
    Key? key,
    required this.photoList,
    this.maxPresentPhoto = DefinedUI.defaultMaxPresentCountGallery,
    this.numberPhotoInBlock = DefinedUI.defaultPhotoCountInGalleryVertBlock,
    this.direction = Axis.vertical,
    this.mainSpacing,
    this.crossSpacing,
    this.padding,
    this.onPickPhoto,
    this.onSeeAll,
    this.convertedSeeAllText,
  })  : assert(
          numberPhotoInBlock > 1,
          'Each block must have more than 1 photo!',
        ),
        super(key: key);

  void _onPickPhoto(BuildContext context, int index) {
    onPickPhoto?.call(index);
    KntGalleryPage.push(
      context,
      photoList: photoList,
      initIndex: index,
    );
  }

  void _onSeeAll(BuildContext context) {
    onSeeAll?.call();
    context.read<PagingGalleryBloc>().add(LoadGalleryPhotosEvent(photoList));
    KntPhotoBrowserPage.push(context);
  }

  @override
  Widget build(BuildContext context) {
    final isVerticalScroll = direction == Axis.vertical;
    var numberHiddenPhoto = photoList.length - maxPresentPhoto;
    final storageType =
        photoList.isEmpty ? StorageType.network : photoList.first.storageType;
    return StaggeredGridView.countBuilder(
      shrinkWrap: true,
      scrollDirection: direction,
      crossAxisCount: numberPhotoInBlock - 1,
      physics: isVerticalScroll
          ? const NeverScrollableScrollPhysics()
          : const BouncingScrollPhysics(),
      padding: padding ?? const EdgeInsets.all(0),
      itemCount: math.min(photoList.length, maxPresentPhoto),
      itemBuilder: (BuildContext context, int index) {
        final photo = photoList[index];
        final hiddenPhotoBackground = SizedBox(
          child: (convertedSeeAllText != null &&
                  numberHiddenPhoto > 1 &&
                  index == maxPresentPhoto - 2)
              ? KntRipple(
                  onTap: () => _onSeeAll(context),
                  child: Container(
                    decoration: BoxDecoration(
                      color: Colors.black54,
                      borderRadius: BorderRadius.circular(16.w),
                    ),
                    alignment: Alignment.center,
                    child: Text(
                      convertedSeeAllText?.call(numberHiddenPhoto) ?? '',
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        color: Colors.white,
                        fontSize: 18.sp,
                        fontWeight: FontWeight.w600,
                        height: 21.sp / 18.sp,
                      ),
                    ),
                  ),
                )
              : const SizedBox(),
        );
        return InkWell(
          onTap: () => _onPickPhoto(context, index),
          child: Hero(
            tag: photo.id,
            child: storageType == StorageType.file
                ? KntLocalImage(
                    relativePath: photo.mediumSizeUri,
                    borderRadius: BorderRadius.circular(16.w),
                    child: hiddenPhotoBackground,
                  )
                : KntCachedImage(
                    url: photo.mediumSizeUri,
                    borderRadius: BorderRadius.circular(16.w),
                    child: hiddenPhotoBackground,
                  ),
          ),
        );
      },
      staggeredTileBuilder: (int index) {
        if (isVerticalScroll) {
          // Each bloc has [numberPhotoInBlock] photos which shared into [_defaultStyleCount] styles
          final bigChunk = index % numberPhotoInBlock == 0 ||
              index % numberPhotoInBlock ==
                  (numberPhotoInBlock - _defaultStyleCount);
          return StaggeredTile.count(bigChunk ? 3 : 2, bigChunk ? 4 : 2);
        }
        final bigChunk = index % numberPhotoInBlock == 0;
        return StaggeredTile.count(bigChunk ? 4 : 1, bigChunk ? 1.6 : 1);
      },
      mainAxisSpacing: mainSpacing ?? 12.w,
      crossAxisSpacing: crossSpacing ?? 12.w,
    );
  }
}
